###############   MORE method   ###############
#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#

library(shiny)
library(shinythemes)
library(MORE)

# Define UI for application 

# Page style
ui <- fluidPage(
  theme = shinythemes::shinytheme("cerulean"),

 # Application title
 titlePanel(
   fluidRow(
     column(8, "Multi-Omic REgulation"),
     column(4, img(src = "logo.PNG", height = 125, width = 225), align = "right"))
 ),
 
 # Choose data
 fluidRow(
   column(12, 
          
          fileInput("datafile", label = "Choose your data set", multiple = FALSE, accept = ".RData"))
  
 ),
 
 # Inputs: data sets, inputs by user...
 fluidRow(
   column(3, uiOutput("geneExpression")),
   column(3, uiOutput("associations")),
   column(3, uiOutput("dataOmics")),
   column(3, uiOutput("eDesign"))
 ),
 
 fluidRow(
   column(3, selectInput("center", 
                         "Centering", 
                         choices = list("TRUE" = TRUE, "FALSE" = FALSE))),
   column(3, selectInput("scale", 
                         "Scaling", 
                         choices = list("TRUE" = TRUE, "FALSE" = FALSE), selected = "FALSE")),
   column(3, numericInput("ResDF", 
                          "Residual degrees of freedom", 
                          value = 5, 
                          min = 1)),
   column(3, numericInput("epsilon", 
                          "Epsilon", 
                          value = 0.00001, 
                          min = 0, 
                          max = 1, 
                          step = 0.00001))
 ), 
 
 fluidRow(
   
   column(3, numericInput("alpha", 
                          "Alpha", 
                          value = 0.05, 
                          min = 0, 
                          max = 1, 
                          step = 0.01)),
   column(3, selectInput("MTadjust", 
                       "Multiple Testing Method", 
                       choices = c("Benjamini and Hochberg" = "fdr",
                                   "Bonferroni" = "bonferroni",
                                   "Holm" = "holm",
                                   "Hochberg" = "hochberg",
                                   "Hommel" = "hommel",
                                   "Benjamini and Yekutieli" = "BY",
                                   "None" = "none"), 
                       selected = "Benjamini and Hochberg")),
   column(3, selectInput("family", 
                         "GLM family",
                         choices = c("Negative Binomial" = "negative.binomial",
                                     "Poisson" = "poisson",
                                     "Quasipoisson" = "quasipoisson",
                                     "Gaussian" = "gaussian",
                                     "Binomial" = "binomial"),
                         selected = "Negative Binomial")),
   column(3, numericInput("elasticnet", 
                          "ElasticNet", 
                          value = 0.5, 
                          min = 0, 
                          max = 1, 
                          step = 0.1))
 ),
 
 fluidRow(
   
   column(3, selectInput("stepwise", 
                         "Stepwise", 
                         choices = list("Backward" = "backward", 
                                        "Forward" = "forward", 
                                        "Two ways backward" = "two.ways.backward", 
                                        "Two ways forward" = "two.ways.forward",
                                        "None" = "none"))),
   column(3, selectInput("InteractionsReg", 
                         "Regulators interactions", 
                         choices = list("TRUE" = TRUE, "FALSE" = FALSE))),
   column(3, textInput('vectorVariation', 
                       "Minimal Variation", 
                       placeholder = "Enter a vector (comma delimited), number or NULL")),
   column(3, numericInput("correlation", 
                          "Correlation", 
                          value = 0.9, 
                          min = 0, 
                          max = 1, 
                          step = 0.1))
 ),
 
 fluidRow(
   
   column(3, numericInput("MinObs", 
                          "Minimal Observations", 
                          value = 10, 
                          min = 1)),
   column(3, textInput('vectorOmicType', "Type of Omics", placeholder = "Enter a vector (comma delimited), number or NULL")),
   column(3, selectInput("betaTest", 
                         "Hypothesis contrast", 
                         choices = list("TRUE" = TRUE, "FALSE" = FALSE)))
 ),


 
 # Button to start the app
 fluidRow(
   column(12, actionButton("goButton", 
                        label = "Start GLM",
                        class = "btn-primary",
                        width = '210px',
                        icon("play-circle")), 
            align = "right")
   ),
 
 # Horizontal line
 tags$hr(),
 
 # Table from RegulationPerCondition
 fluidRow(
   dataTableOutput("tableMORE"),
   column(6, uiOutput("viewButton"), align = "left"),
   column(6, uiOutput("tableDownload"), align = "right")
 ),
 
 tags$hr(),
 # Plots and inputs plotGLM
 

 fluidRow(conditionalPanel(
     condition = "input.plotConfig > 0",
     column(3, textInput("gene", "Gene", placeholder = "Enter a gene or NULL")), 
     column(3, textInput("regulator", "Regulator", placeholder = "Enter a regulator or NULL")),
     column(3, textInput('reguValues', "Values Regulator", placeholder = "Enter a vector (comma delimited) or NULL")),
     column(3, selectInput("plotPerOmic", "Plot Per Omic", choices = list("TRUE" = TRUE, "FALSE" = FALSE)))
     
     )),
 
 fluidRow(conditionalPanel(
    condition = "input.plotConfig > 0",
    column(3, selectInput("geneCol", "Gene Colour", choices = list("Black" = "black", 
                                                                   "Green" = "palegreen4", 
                                                                   "Blue" = "royalblue3", 
                                                                   "Red" = "tomato2",
                                                                   "Purple" = "darkorchid3",
                                                                   "Orange" = "tan1",
                                                                   "Yellow" = "yellow2",
                                                                   "Grey" = "snow3"))),
    column(3, selectInput("reguCol", "Regulator Colour", choices = list("Green" = "palegreen4", 
                                                                        "Blue" = "royalblue3", 
                                                                        "Red" = "tomato2",
                                                                        "Black" = "black",
                                                                        "Purple" = "darkorchid3",
                                                                        "Orange" = "tan1",
                                                                        "Yellow" = "yellow2",
                                                                        "Grey" = "snow3"))),
    column(3,selectInput("order", "Order", choices = list("TRUE" = TRUE, "FALSE" = FALSE))),
    column(3, textInput("xlab", "Title X axis", value = ""))
    
    )),
        
 fluidRow(conditionalPanel(
    condition = "input.plotConfig > 0",
    column(3, textInput('ContVar', "Values Continuous Variable", placeholder = "Enter a vector (comma delimited) or NULL")),
    column(3, textInput('cond2plot', "Condition to plot", placeholder = "Enter a vector (comma delimited) or NULL"))
    
    )),
 
 fluidRow(conditionalPanel(
    condition = "input.plotConfig > 0",
    column(6, actionButton("generatePlot", label = "Generate Plot", class = "btn-primary", width = '250px'), align = "left"),
    column(6, downloadButton('downloadPlot'), align = "right")
    
    )),
 
 fluidRow(column(12, plotOutput(outputId = "plotMORE"))), # simple case
   # column(8, uiOutput("plots")) # try generate all plots
 
 tags$hr()

)

# For large files
options(shiny.maxRequestSize = 30*1024*1024^2)

# Define server logic required to draw a histogram
server <- function(input, output) {
  
  # For choices in set data
  infile <- reactive({
    infile <- input$datafile
    if (is.null(infile)) {
      return(NULL)
    }
    objectsLoaded <- load(input$datafile$datapath, envir = .GlobalEnv) 
    return(objectsLoaded)
  })
  
  myData <- reactive({
    df<-infile()
    if (is.null(df)) return(NULL)
    return(df)
  })
  
  # Select data sets
  output$geneExpression <- renderUI(
    selectInput("geneExpression", "Gene Expression", choices = myData()))
  
  output$associations <- renderUI(
    selectInput("associations", "Associations", choices = myData()))
  
  output$dataOmics <- renderUI(
    selectInput("dataOmics", "Regulatory omic data", choices = myData()))
  
  output$eDesign <- renderUI(
    selectInput("eDesign", "Experimental design matrix", choices = myData()))
  
  OutPutGLM = list()
  
  observeEvent(input$goButton, {
    
    # Inputs can be NULL
    # contiVar <- reactive({
    #   if (identical(input$contVar, ""))
    #     NULL
    #   else 
    #     input$contVar
    # })
    
    
    # OmicType and min.variation names
    if(nchar(input$vectorOmicType) > 1){
      omicType = as.numeric(unlist(strsplit(input$vectorOmicType, ",")))
    } else if(nchar(input$vectorOmicType) == 1){
      omicType = as.numeric(input$vectorOmicType)
    }
    names(omicType) = names(input$dataOmics)
    
    
    if(nchar(input$vectorVariation) > 1) {
      minVar = as.numeric(unlist(strsplit(input$vectorVariation, ",")))
      names(minVar) = names(input$dataOmics)
      
    } else if(length(input$vectorVariation == 1)){
      minVar = as.numeric(input$vectorVariation)
    } 
    
    MinimalVariation <- reactive({
      if (identical(input$vectorVariation, ""))
        NULL
      else 
        input$vectorVariation
    })
    
    # Family GLM
    myFamily <- reactive({
      if(input$family == "negative.binomial"){
         negative.binomial(theta = 10)
      } else if(input$family == "poisson"){
        poisson()
      } else if(input$family == "quasipoisson"){
        quasipoisson()
      } else if(input$family == "gaussian"){
        gaussian()
      } else if(input$family == "binomial"){
        binomial()
      }
    })
   
    # GetGLM function
    OutPutGLM <<- GetGLM(GeneExpression = eval(parse(text=input$geneExpression)),
                    associations = eval(parse(text=input$associations)),
                    data.omics = eval(parse(text=input$dataOmics)),
                    edesign = eval(parse(text=input$eDesign)),
                    Res.df = input$ResDF, 
                    epsilon = input$epsilon,
                    alfa = input$alpha, 
                    MT.adjust = input$MTadjust,
                    family = myFamily(),
                    elasticnet = input$elasticnet,
                    stepwise = input$stepwise, 
                    interactions.reg = input$InteractionsReg,
                    correlation = input$correlation,
                    min.variation = MinimalVariation(),
                    min.obs = input$MinObs, 
                    omic.type = omicType)
    
    # Table
    myResults = RegulationPerCondition(OutPutGLM, input$betaTest)
    output$tableMORE = renderDataTable(myResults)
    
    output$downloadTable = downloadHandler(
      filename = function(){paste("TableMORE.csv")},
      content = function(file){
        write.csv(myResults, file)
      }
    )
    
    output$tableDownload = renderUI({downloadButton("downloadTable", "Download")})

    output$viewButton = renderUI(actionButton("plotConfig", 
                                              label = "MORE plots", 
                                              class = "btn-primary",
                                              width = '250px'))
    
  })

  
  # Plot
  observeEvent(input$generatePlot, {
    
    # Inputs can be NULL
    regulatorValues <- reactive({
      if (identical(input$reguValues, ""))
        NULL
      else if(nchar(input$reguValues) > 1)
        ReguValues = as.numeric(unlist(strsplit(input$reguValues, ",")))
    })
    
    
    ContinuousVar <- reactive({
      if (identical(input$ContVar, ""))
        NULL  
      else if(nchar(input$ContVar) > 1)
        contVar = as.numeric(unlist(strsplit(input$ContVar, ",")))
    })
    
    
    CondToPlot <- reactive({
      if (identical(input$cond2plot, ""))
        NULL
      else 
        input$cond2plot
    })
    
    Gene <- reactive({
      if (identical(input$gene, ""))
        NULL
      else 
        input$gene
    })
    
    Regulator <- reactive({
      if (identical(input$regulator, ""))
        NULL
      else 
        input$regulator
    })
    
    # Simple case
    
    output$plotMORE = renderPlot({
      plotGLM(OutPutGLM, 
              Gene(), 
              Regulator(), 
              regulatorValues(),
              input$plotPerOmic, 
              input$geneCol, 
              input$reguCol,
              input$order, 
              input$xlab, 
              ContinuousVar(), 
              CondToPlot())
    })
    
    # Save images in a pdf
    output$downloadPlot = downloadHandler(
      filename = function(){paste0("plotsGLM.pdf")},
      content = function(file){
        pdf(file, onefile = TRUE)
        
        print(plotGLM(OutPutGLM, 
                      Gene(), 
                      Regulator(), 
                      regulatorValues(),
                      input$plotPerOmic, 
                      input$geneCol, 
                      input$reguCol,
                      input$order, 
                      input$xlab, 
                      ContinuousVar(), 
                      CondToPlot()))
        
        dev.off()
        })

    
  
    
    ## Try generate all plots
    
    # listPlots <<- plotGLM(OutPutGLM, Gene(), Regulator(), regulatorValues(),
    #                       input$plotPerOmic, input$geneCol, input$reguCol,
    #                       input$order, input$xlab, ContinuousVar(), CondToPlot())
    # 
    # output$plots <- renderUI({
    #   plot_output_list <- lapply(1:length(listPlots), function(i) {
    #     plotname <- paste("plot", i, sep="")
    #     plotOutput(plotname)
    #   })
    # 
    #   do.call(tagList, plot_output_list)
    # })
    # 
    # 
    # for (i in 1:length(listPlots)) {
    # 
    #   local({
    #     my_i <- i
    #     plotname <- paste("plot", my_i, sep="")
    # 
    #     output[[plotname]] <- renderPlot({listPlots[i]})
    #   })
    # }

})
  
}


# Run the application 
shinyApp(ui = ui, server = server)

